@@ -341,11 +341,14 @@ HRESULT CInArchive::ReadLocalItemAfterCdItem(CItemEx &item)
     RINOK(ReadLocalItem(localItem));
     if (item.Flags != localItem.Flags)
     {
-      if ((item.CompressionMethod != NFileHeader::NCompressionMethod::kDeflated ||
-        (item.Flags & 0x7FFC) != (localItem.Flags & 0x7FFC)) &&
-        ((item.CompressionMethod != NFileHeader::NCompressionMethod::kStored ||
-        (item.Flags & 0x7FFF) != (localItem.Flags & 0x7FFF))
-        ))
+      if (
+          (item.CompressionMethod != NFileHeader::NCompressionMethod::kDeflated ||
+            (item.Flags & 0x7FF9) != (localItem.Flags & 0x7FF9)) &&
+          (item.CompressionMethod != NFileHeader::NCompressionMethod::kStored ||
+            (item.Flags & 0x7FFF) != (localItem.Flags & 0x7FFF)) &&
+          (item.CompressionMethod != NFileHeader::NCompressionMethod::kImploded ||
+            (item.Flags & 0x7FFF) != (localItem.Flags & 0x7FFF))
+        )
         return S_FALSE;
     }
 
@@ -539,6 +542,10 @@ HRESULT CInArchive::FindCd(CCdInfo &cdInfo)
         // cdInfo.NumEntries = GetUInt16(buf + i + 10);
         cdInfo.Size = GetUInt32(buf + i + 12);
         cdInfo.Offset = GetUInt32(buf + i + 16);
+        UInt64 curPos = endPosition - bufSize + i;
+        UInt64 cdEnd = cdInfo.Size + cdInfo.Offset;
+        if (curPos > cdEnd)
+          m_ArchiveInfo.Base = curPos - cdEnd;
         return S_OK;
       }
     }
