@@ -611,14 +611,14 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
   {
     reps[i] = _repDistances[i];
     UInt32 backOffset = reps[i] + 1;
-    if (data[0] != data[(size_t)0 - backOffset] || data[1] != data[(size_t)1 - backOffset])
+    if (data[0] != data[(ptrdiff_t)0 - backOffset] || data[1] != data[(ptrdiff_t)1 - backOffset])
     {
       repLens[i] = 0;
       continue;
     }
     UInt32 lenTest;
     for (lenTest = 2; lenTest < numAvailableBytes && 
-        data[lenTest] == data[(size_t)lenTest - backOffset]; lenTest++);
+        data[lenTest] == data[(ptrdiff_t)lenTest - backOffset]; lenTest++);
     repLens[i] = lenTest;
     if (lenTest > repLens[repMaxIndex])
       repMaxIndex = i;
@@ -639,7 +639,7 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
     return lenMain;
   }
   Byte currentByte = *data;
-  Byte matchByte = data[(size_t)0 - reps[0] - 1];
+  Byte matchByte = data[(ptrdiff_t)0 - reps[0] - 1];
 
   if(lenMain < 2 && currentByte != matchByte && repLens[repMaxIndex] < 2)
   {
@@ -821,13 +821,13 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
     UInt32 curPrice = curOptimum.Price; 
     const Byte *data = _matchFinder.GetPointerToCurrentPos(_matchFinderObj) - 1;
     const Byte currentByte = *data;
-    const Byte matchByte = data[(size_t)0 - reps[0] - 1];
+    const Byte matchByte = data[(ptrdiff_t)0 - reps[0] - 1];
 
     UInt32 posState = (position & _posStateMask);
 
     UInt32 curAnd1Price = curPrice +
         _isMatch[state.Index][posState].GetPrice0() +
-        _literalEncoder.GetSubCoder(position, data[(size_t)0 - 1])->GetPrice(!state.IsCharState(), matchByte, currentByte);
+        _literalEncoder.GetSubCoder(position, data[(ptrdiff_t)0 - 1])->GetPrice(!state.IsCharState(), matchByte, currentByte);
 
     COptimal &nextOptimum = _optimum[cur + 1];
 
@@ -874,7 +874,7 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
       UInt32 limit = MyMin(numAvailableBytesFull, _numFastBytes + 1);
       UInt32 temp;
       for (temp = 1; temp < limit && 
-          data[temp] == data[(size_t)temp - backOffset]; temp++);
+          data[temp] == data[(ptrdiff_t)temp - backOffset]; temp++);
       UInt32 lenTest2 = temp - 1;
       if (lenTest2 >= 2)
       {
@@ -909,12 +909,12 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
     {
       // UInt32 repLen = _matchFinder.GetMatchLen(0 - 1, reps[repIndex], newLen); // test it;
       UInt32 backOffset = reps[repIndex] + 1;
-      if (data[0] != data[(size_t)0 - backOffset] ||
-          data[1] != data[(size_t)1 - backOffset])
+      if (data[0] != data[(ptrdiff_t)0 - backOffset] ||
+          data[1] != data[(ptrdiff_t)1 - backOffset])
         continue;
       UInt32 lenTest;
       for (lenTest = 2; lenTest < numAvailableBytes && 
-          data[lenTest] == data[(size_t)lenTest - backOffset]; lenTest++);
+          data[lenTest] == data[(ptrdiff_t)lenTest - backOffset]; lenTest++);
       while(lenEnd < cur + lenTest)
         _optimum[++lenEnd].Price = kIfinityPrice;
       UInt32 lenTestTemp = lenTest;
@@ -942,7 +942,7 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
           UInt32 lenTest2 = lenTest + 1;
           UInt32 limit = MyMin(numAvailableBytesFull, lenTest2 + _numFastBytes);
           for (; lenTest2 < limit && 
-              data[lenTest2] == data[(size_t)lenTest2 - backOffset]; lenTest2++);
+              data[lenTest2] == data[(ptrdiff_t)lenTest2 - backOffset]; lenTest2++);
           lenTest2 -= lenTest + 1;
           if (lenTest2 >= 2)
           {
@@ -952,8 +952,8 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
             UInt32 curAndLenCharPrice = 
                 price + _repMatchLenEncoder.GetPrice(lenTest - 2, posState) + 
                 _isMatch[state2.Index][posStateNext].GetPrice0() +
-                _literalEncoder.GetSubCoder(position + lenTest, data[(size_t)lenTest - 1])->GetPrice(
-                true, data[(size_t)lenTest - backOffset], data[lenTest]);
+                _literalEncoder.GetSubCoder(position + lenTest, data[(ptrdiff_t)lenTest - 1])->GetPrice(
+                true, data[(ptrdiff_t)lenTest - backOffset], data[lenTest]);
             state2.UpdateChar();
             posStateNext = (position + lenTest + 1) & _posStateMask;
             UInt32 nextRepMatchPrice = curAndLenCharPrice + 
@@ -1029,7 +1029,7 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
           UInt32 lenTest2 = lenTest + 1;
           UInt32 limit = MyMin(numAvailableBytesFull, lenTest2 + _numFastBytes);
           for (; lenTest2 < limit && 
-              data[lenTest2] == data[(size_t)lenTest2 - backOffset]; lenTest2++);
+              data[lenTest2] == data[(ptrdiff_t)lenTest2 - backOffset]; lenTest2++);
           lenTest2 -= lenTest + 1;
           if (lenTest2 >= 2)
           {
@@ -1038,8 +1038,8 @@ UInt32 CEncoder::GetOptimum(UInt32 position, UInt32 &backRes)
             UInt32 posStateNext = (position + lenTest) & _posStateMask;
             UInt32 curAndLenCharPrice = curAndLenPrice + 
                 _isMatch[state2.Index][posStateNext].GetPrice0() +
-                _literalEncoder.GetSubCoder(position + lenTest, data[(size_t)lenTest - 1])->GetPrice( 
-                true, data[(size_t)lenTest - backOffset], data[lenTest]);
+                _literalEncoder.GetSubCoder(position + lenTest, data[(ptrdiff_t)lenTest - 1])->GetPrice( 
+                true, data[(ptrdiff_t)lenTest - backOffset], data[lenTest]);
             state2.UpdateChar();
             posStateNext = (posStateNext + 1) & _posStateMask;
             UInt32 nextRepMatchPrice = curAndLenCharPrice + 
@@ -1105,7 +1105,7 @@ UInt32 CEncoder::ReadMatchDistances(UInt32 &numDistancePairs)
       UInt32 distance = _matchDistances[numDistancePairs - 1] + 1;
       if (numAvail > kMatchMaxLen)
         numAvail = kMatchMaxLen;
-      for (; lenRes < numAvail && pby[lenRes] == pby[(size_t)lenRes - distance]; lenRes++);
+      for (; lenRes < numAvail && pby[lenRes] == pby[(ptrdiff_t)lenRes - distance]; lenRes++);
     }
   }
   _additionalOffset++;
@@ -1142,13 +1142,13 @@ UInt32 CEncoder::GetOptimumFast(UInt32 &backRes)
   for(UInt32 i = 0; i < kNumRepDistances; i++)
   {
     UInt32 backOffset = _repDistances[i] + 1;
-    if (data[0] != data[(size_t)0 - backOffset] || data[1] != data[(size_t)1 - backOffset])
+    if (data[0] != data[(ptrdiff_t)0 - backOffset] || data[1] != data[(ptrdiff_t)1 - backOffset])
     {
       repLens[i] = 0;
       continue;
     }
     UInt32 len;
-    for (len = 2; len < numAvailableBytes && data[len] == data[(size_t)len - backOffset]; len++);
+    for (len = 2; len < numAvailableBytes && data[len] == data[(ptrdiff_t)len - backOffset]; len++);
     if(len >= _numFastBytes)
     {
       backRes = i;
@@ -1217,13 +1217,13 @@ UInt32 CEncoder::GetOptimumFast(UInt32 &backRes)
     for(UInt32 i = 0; i < kNumRepDistances; i++)
     {
       UInt32 backOffset = _repDistances[i] + 1;
-      if (data[1] != data[(size_t)1 - backOffset] || data[2] != data[(size_t)2 - backOffset])
+      if (data[1] != data[(ptrdiff_t)1 - backOffset] || data[2] != data[(ptrdiff_t)2 - backOffset])
       {
         repLens[i] = 0;
         continue;
       }
       UInt32 len;
-      for (len = 2; len < numAvailableBytes && data[len] == data[(size_t)len - backOffset]; len++);
+      for (len = 2; len < numAvailableBytes && data[len] == data[(ptrdiff_t)len - backOffset]; len++);
       if (len + 1 >= lenMain)
       {
         _longestMatchWasFound = true;
